@@ -396,7 +396,7 @@ public class Stream extends StructureObject {
     SortedMap<String, Charset> available = Charset.availableCharsets();
     Set<String> encodings = available.keySet();
     for (String charset : encodings) {
-      result.add(new Symbol(charset, PACKAGE_KEYWORD));
+      result.add (PACKAGE_KEYWORD.intern (charset));
     }
     return result;
   }
@@ -440,7 +440,7 @@ public class Stream extends StructureObject {
     }
 
     // Character input.
-    public final int getLineNumber() {
+    public int getLineNumber() {
         return lineNumber;
     }
 
@@ -515,6 +515,7 @@ public class Stream extends StructureObject {
                 return error(new StreamError(this, e));
             }
         }
+        if (!eofError && result == eofValue) return result;
         if (Symbol.READ_SUPPRESS.symbolValue(thread) != NIL)
             return NIL;
         else
@@ -855,7 +856,7 @@ public class Stream extends StructureObject {
         StringBuilder sb = new StringBuilder();
         try 
             {
-		while (true) {
+                while (true) {
                     int ch = _readChar();
                     if (ch < 0)
                         break;
@@ -876,13 +877,13 @@ public class Stream extends StructureObject {
                                                   this));
                         }
                     }
-		}
+                }
             }
         catch (java.io.IOException e)
             {
-		error(new ReaderError("IO error: ",
-				      this));
-		return NIL;
+                error(new ReaderError("IO error: ",
+                                      this));
+                return NIL;
             }
         
         if (suppress)
@@ -1190,16 +1191,22 @@ public class Stream extends StructureObject {
                     return symbol;
 
                 // Error!
-                if (pkg.findInternalSymbol(symbolName) != null)
-                    return error(new ReaderError("The symbol \"" + symbolName +
-                                                 "\" is not external in package " +
-                                                 packageName + '.',
-                                                 this));
-                else
-                    return error(new ReaderError("The symbol \"" + symbolName +
-                                                 "\" was not found in package " +
-                                                 packageName + '.',
-                                                 this));
+                if (pkg.findInternalSymbol(symbolName) != null) {
+                    return error(new ReaderError("The symbol \"~A\" is not external in package ~A.",
+                                                 this,
+                                                 new SimpleString(symbolName),
+                                                 new SimpleString(packageName)));
+                } else {
+                  if (packageName.equals("KEYWORD")) {
+                    return pkg.intern(symbolName);
+                  }
+                  else {
+                    return error(new ReaderError("The symbol \"~A\" was not found in package ~A.",
+                                                 this,
+                                                 new SimpleString(symbolName),
+                                                 new SimpleString(packageName)));
+                  }
+                }
             }
         } else {                // token.length == 0
             Package pkg = (Package)Symbol._PACKAGE_.symbolValue(thread);
@@ -1691,6 +1698,11 @@ public class Stream extends StructureObject {
         return NIL;
     }
 
+    /**
+     Emit a newline unless at character position zero
+     
+     Return T if newline was emitted, NIL otherwise
+    */
     public LispObject freshLine() {
         if (charPos == 0)
             return NIL;
@@ -2239,10 +2251,18 @@ public class Stream extends StructureObject {
     };
 
     // ### out-synonym-of stream-designator => stream
-    private static final Primitive OUT_SYNONYM_OF =
-    new Primitive("out-synonym-of", PACKAGE_SYS, true, "stream-designator") {
-        @Override
+    public static final Primitive OUT_SYNONYM_OF
+        = new pf_out_synonym_of();
+    private static final class pf_out_synonym_of extends Primitive  {
+        pf_out_synonym_of() {
+            super("out-synonym-of", PACKAGE_SYS, true, "stream-designator");
+        }
         public LispObject execute (LispObject arg) {
+            if (arg instanceof SynonymStream) {
+                Symbol symbol
+                    = (Symbol) SynonymStream.SYNONYM_STREAM_SYMBOL.execute(arg);
+                return symbol.symbolValue();
+            }
             if (arg instanceof Stream)
                 return arg;
             if (arg == T)
@@ -2665,19 +2685,18 @@ public class Stream extends StructureObject {
     };
 
     public InputStream getWrappedInputStream() {
-	return in;
+        return in;
     }
 
     public OutputStream getWrappedOutputStream() {
-	return out;
+        return out;
     }
 
     public Writer getWrappedWriter() {
-	return writer;
+        return writer;
     }
 
     public PushbackReader getWrappedReader() {
-	return reader;
+        return reader;
     }
-
 }
